@@ -6,12 +6,13 @@
 
     :copyright: (c) 2012 by Matt Wright.
     :copyright: (c) 2017 by CERN.
+    :copyright: (c) 2021 by J. Christopher Wagner (jwag).
     :license: MIT, see LICENSE for more details.
 """
 
 from flask import current_app as app
-from werkzeug.local import LocalProxy
 
+from .proxies import _security, _datastore
 from .signals import confirm_instructions_sent, user_confirmed
 from .utils import (
     config_value,
@@ -22,11 +23,6 @@ from .utils import (
     verify_hash,
 )
 
-# Convenient references
-_security = LocalProxy(lambda: app.extensions["security"])
-
-_datastore = LocalProxy(lambda: _security.datastore)
-
 
 def generate_confirmation_link(user):
     token = generate_confirmation_token(user)
@@ -47,9 +43,12 @@ def send_confirmation_instructions(user):
         "confirmation_instructions",
         user=user,
         confirmation_link=confirmation_link,
+        confirmation_token=token,
     )
 
-    confirm_instructions_sent.send(app._get_current_object(), user=user, token=token)
+    confirm_instructions_sent.send(
+        app._get_current_object(), user=user, token=token, confirmation_token=token
+    )
 
 
 def generate_confirmation_token(user):
