@@ -570,7 +570,7 @@ class SQLAlchemyUserDatastore(SQLAlchemyDatastore, UserDatastore):
         if config_value("JOIN_USER_ROLES") and hasattr(self.user_model, "roles"):
             from sqlalchemy.orm import joinedload
 
-            query = query.options(joinedload(self.user_model.roles))
+            query = query.options(joinedload(self.user_model.roles))  # type: ignore
 
         if case_insensitive:
             # While it is of course possible to pass in multiple keys to filter on
@@ -613,7 +613,10 @@ class SQLAlchemySessionUserDatastore(SQLAlchemyUserDatastore, SQLAlchemyDatastor
                 self.session = session
 
         SQLAlchemyUserDatastore.__init__(
-            self, PretendFlaskSQLAlchemyDb(session), user_model, role_model
+            self,
+            PretendFlaskSQLAlchemyDb(session),  # type: ignore
+            user_model,
+            role_model,
         )
 
     def commit(self):
