@@ -5,7 +5,7 @@
     Flask-Security views module
 
     :copyright: (c) 2012 by Matt Wright.
-    :copyright: (c) 2019-2020 by J. Christopher Wagner (jwag).
+    :copyright: (c) 2019-2021 by J. Christopher Wagner (jwag).
     :license: MIT, see LICENSE for more details.
 
     CSRF is tricky. By default all our forms have CSRF protection built in via
@@ -26,24 +26,21 @@
     so app must set CSRF_IGNORE_UNAUTH_ENDPOINTS (or use CSRF/session cookie for logging
     in then once they have a token, no need for cookie).
 
-    TODO: two-factor routes such as tf_setup need work. They seem to support both
-    authenticated (via session?) as well as unauthenticated access.
 """
 
 from functools import partial
 import time
+import typing as t
 
 from flask import (
     Blueprint,
     after_this_request,
-    current_app,
     jsonify,
     request,
     session,
 )
 from flask_login import current_user
-from werkzeug.datastructures import MultiDict
-from werkzeug.local import LocalProxy
+from werkzeug.datastructures import ImmutableMultiDict, MultiDict
 
 from .changeable import change_user_password
 from .confirmable import (
@@ -53,6 +50,7 @@ from .confirmable import (
 )
 from .decorators import anonymous_user_required, auth_required, unauth_csrf
 from .passwordless import login_token_status, send_login_instructions
+from .proxies import _security, _datastore
 from .quart_compat import get_quart_status
 from .unified_signin import (
     us_signin,
@@ -82,7 +80,7 @@ from .twofactor import (
 from .utils import (
     base_render_json,
     check_and_update_authn_fresh,
-    config_value,
+    config_value as cv,
     do_flash,
     get_message,
     get_post_login_redirect,
@@ -106,10 +104,8 @@ if get_quart_status():  # pragma: no cover
 else:
     from flask import make_response, redirect
 
-
-# Convenient references
-_security = LocalProxy(lambda: current_app.extensions["security"])
-_datastore = LocalProxy(lambda: _security.datastore)
+if t.TYPE_CHECKING:  # pragma: no cover
+    from flask.typing import ResponseValue
 
 
 def default_render_json(payload, code, headers, user):
@@ -127,7 +123,7 @@ def _ctx(endpoint):
 
 
 @unauth_csrf(fall_through=True)
-def login():
+def login() -> "ResponseValue":
     """View function for login view
 
     Allow already authenticated users. For GET this is useful for
@@ -150,7 +146,7 @@ def login():
             )
             return _security._render_json(payload, 400, None, None)
         else:
-            return redirect(get_url(_security.post_login_view))
+            return redirect(get_url(cv("POST_LOGIN_VIEW")))
 
     form_class = _security.login_form
 
@@ -165,9 +161,11 @@ def login():
 
     if form.validate_on_submit():
         remember_me = form.remember.data if "remember" in form else None
-        if config_value("TWO_FACTOR"):
+        if cv("TWO_FACTOR"):
             if request.is_json and request.content_length:
-                tf_validity_token = request.get_json().get("tf_validity_token", None)
+                tf_validity_token = request.get_json().get(  # type: ignore
+                    "tf_validity_token", None
+                )
             else:
                 tf_validity_token = request.cookies.get("tf_validity", default=None)
 
@@ -175,11 +173,8 @@ def login():
                 tf_validity_token, form.user.fs_uniquifier
             )
 
-            if config_value("TWO_FACTOR_REQUIRED") or (is_tf_setup(form.user)):
-                if config_value("TWO_FACTOR_ALWAYS_VALIDATE") or (
-                    not tf_validity_token_is_valid
-                ):
-
+            if cv("TWO_FACTOR_REQUIRED") or (is_tf_setup(form.user)):
+                if cv("TWO_FACTOR_ALWAYS_VALIDATE") or (not tf_validity_token_is_valid):
                     return tf_login(
                         form.user, remember=remember_me, primary_authn_via="password"
                     )
@@ -197,22 +192,22 @@ def login():
         return base_render_json(form)
 
     if current_user.is_authenticated:
-        return redirect(get_url(_security.post_login_view))
+        return redirect(get_url(cv("POST_LOGIN_VIEW")))
     else:
-        if form.requires_confirmation and _security.requires_confirmation_error_view:
+        if form.requires_confirmation and cv("REQUIRES_CONFIRMATION_ERROR_VIEW"):
             do_flash(*get_message("CONFIRMATION_REQUIRED"))
             return redirect(
                 get_url(
-                    _security.requires_confirmation_error_view,
+                    cv("REQUIRES_CONFIRMATION_ERROR_VIEW"),
                     qparams={"email": form.email.data},
                 )
             )
         return _security.render_template(
-            config_value("LOGIN_USER_TEMPLATE"), login_user_form=form, **_ctx("login")
+            cv("LOGIN_USER_TEMPLATE"), login_user_form=form, **_ctx("login")
         )
 
 
-@auth_required(lambda: config_value("API_ENABLED_METHODS"))
+@auth_required(lambda: cv("API_ENABLED_METHODS"))
 def verify():
     """View function which handles a authentication verification request."""
     form_class = _security.verify_form
@@ -239,7 +234,7 @@ def verify():
         return base_render_json(form)
 
     return _security.render_template(
-        config_value("VERIFY_TEMPLATE"), verify_form=form, **_ctx("verify")
+        cv("VERIFY_TEMPLATE"), verify_form=form, **_ctx("verify")
     )
 
 
@@ -258,7 +253,7 @@ def logout():
 
 
 @anonymous_user_required
-def register():
+def register() -> "ResponseValue":
     """View function which handles a registration request."""
 
     # For some unknown historic reason - if you don't require confirmation
@@ -271,7 +266,7 @@ def register():
         form_class = _security.register_form
 
     if request.is_json:
-        form_data = MultiDict(request.get_json())
+        form_data: ImmutableMultiDict = ImmutableMultiDict(request.get_json())
     else:
         form_data = request.form
 
@@ -285,8 +280,8 @@ def register():
         # an application that would want random email accounts. It has been like this
         # since the beginning. Note that we still enforce 2FA - however for unified
         # signin - we adhere to historic behavior.
-        if not _security.confirmable or _security.login_without_confirmation:
-            if config_value("TWO_FACTOR") and config_value("TWO_FACTOR_REQUIRED"):
+        if not _security.confirmable or cv("LOGIN_WITHOUT_CONFIRMATION"):
+            if cv("TWO_FACTOR") and cv("TWO_FACTOR_REQUIRED"):
                 return tf_login(user, primary_authn_via="register")
             after_this_request(view_commit)
             login_user(user, authn_via=["register"])
@@ -301,7 +296,7 @@ def register():
         return base_render_json(form)
 
     return _security.render_template(
-        config_value("REGISTER_USER_TEMPLATE"),
+        cv("REGISTER_USER_TEMPLATE"),
         register_user_form=form,
         **_ctx("register"),
     )
@@ -327,7 +322,7 @@ def send_login():
         return base_render_json(form)
 
     return _security.render_template(
-        config_value("SEND_LOGIN_TEMPLATE"), send_login_form=form, **_ctx("send_login")
+        cv("SEND_LOGIN_TEMPLATE"), send_login_form=form, **_ctx("send_login")
     )
 
 
@@ -393,7 +388,7 @@ def send_confirmation():
         return base_render_json(form)
 
     return _security.render_template(
-        config_value("SEND_CONFIRMATION_TEMPLATE"),
+        cv("SEND_CONFIRMATION_TEMPLATE"),
         send_confirmation_form=form,
         **_ctx("send_confirmation"),
     )
@@ -446,12 +441,12 @@ def confirm_email(token):
 
     if user != current_user:
         logout_user()
-        if config_value("AUTO_LOGIN_AFTER_CONFIRM"):
+        if cv("AUTO_LOGIN_AFTER_CONFIRM"):
             # N.B. this is a (small) security risk if email went to wrong place.
             # and you have the LOGIN_WITH_CONFIRMATION flag since in that case
             # you can be logged in and doing stuff - but another person could
             # get the email.
-            if config_value("TWO_FACTOR") and config_value("TWO_FACTOR_REQUIRED"):
+            if cv("TWO_FACTOR") and cv("TWO_FACTOR_REQUIRED"):
                 return tf_login(user, primary_authn_via="confirm")
             login_user(user, authn_via=["confirm"])
 
@@ -467,7 +462,7 @@ def confirm_email(token):
         get_url(_security.post_confirm_view)
         or get_url(
             _security.post_login_view
-            if config_value("AUTO_LOGIN_AFTER_CONFIRM")
+            if cv("AUTO_LOGIN_AFTER_CONFIRM")
             else _security.login_url
         )
     )
@@ -503,7 +498,7 @@ def forgot_password():
         )
 
     return _security.render_template(
-        config_value("FORGOT_PASSWORD_TEMPLATE"),
+        cv("FORGOT_PASSWORD_TEMPLATE"),
         forgot_password_form=form,
         **_ctx("forgot_password"),
     )
@@ -570,7 +565,7 @@ def reset_password(token):
             )
         # for forms - render the reset password form
         return _security.render_template(
-            config_value("RESET_PASSWORD_TEMPLATE"),
+            cv("RESET_PASSWORD_TEMPLATE"),
             reset_password_form=form,
             reset_password_token=token,
             **_ctx("reset_password"),
@@ -603,8 +598,8 @@ def reset_password(token):
     if form.validate_on_submit():
         after_this_request(view_commit)
         update_password(user, form.password.data)
-        if config_value("TWO_FACTOR") and (
-            config_value("TWO_FACTOR_REQUIRED")
+        if cv("TWO_FACTOR") and (
+            cv("TWO_FACTOR_REQUIRED")
             or (form.user.tf_totp_secret and form.user.tf_primary_method)
         ):
             return tf_login(user, primary_authn_via="reset")
@@ -624,14 +619,14 @@ def reset_password(token):
     if _security._want_json(request):
         return base_render_json(form)
     return _security.render_template(
-        config_value("RESET_PASSWORD_TEMPLATE"),
+        cv("RESET_PASSWORD_TEMPLATE"),
         reset_password_form=form,
         reset_password_token=token,
         **_ctx("reset_password"),
     )
 
 
-@auth_required(lambda: config_value("API_ENABLED_METHODS"))
+@auth_required(lambda: cv("API_ENABLED_METHODS"))
 def change_password():
     """View function which handles a change password request."""
 
@@ -659,7 +654,7 @@ def change_password():
         return base_render_json(form)
 
     return _security.render_template(
-        config_value("CHANGE_PASSWORD_TEMPLATE"),
+        cv("CHANGE_PASSWORD_TEMPLATE"),
         change_password_form=form,
         **_ctx("change_password"),
     )
@@ -713,12 +708,12 @@ def two_factor_setup():
     else:
         # Caller is changing their TFA profile. This requires a 'fresh' authentication
         if not check_and_update_authn_fresh(
-            config_value("FRESHNESS"),
-            config_value("FRESHNESS_GRACE_PERIOD"),
+            cv("FRESHNESS"),
+            cv("FRESHNESS_GRACE_PERIOD"),
             get_request_attr("fs_authn_via"),
         ):
             return _security._reauthn_handler(
-                config_value("FRESHNESS"), config_value("FRESHNESS_GRACE_PERIOD")
+                cv("FRESHNESS"), cv("FRESHNESS_GRACE_PERIOD")
             )
         user = current_user
 
@@ -751,7 +746,9 @@ def two_factor_setup():
             "tf_state": "validating_profile",
             "tf_primary_method": pm,
         }
-        new_phone = form.phone.data if len(form.phone.data) > 0 else None
+        new_phone = (
+            form.phone.data if (form.phone.data and len(form.phone.data) > 0) else None
+        )
         if new_phone:
             user.tf_phone_number = new_phone
             _datastore.put(user)
@@ -795,10 +792,10 @@ def two_factor_setup():
         code_form = _security.two_factor_verify_code_form()
         if not _security._want_json(request):
             return _security.render_template(
-                config_value("TWO_FACTOR_SETUP_TEMPLATE"),
+                cv("TWO_FACTOR_SETUP_TEMPLATE"),
                 two_factor_setup_form=form,
                 two_factor_verify_code_form=code_form,
-                choices=config_value("TWO_FACTOR_ENABLED_METHODS"),
+                choices=cv("TWO_FACTOR_ENABLED_METHODS"),
                 chosen_method=pm,
                 **qrcode_values,
                 **_ctx("tf_setup"),
@@ -808,14 +805,14 @@ def two_factor_setup():
     # We get here on GET and POST with failed validation.
     # For things like phone number - we've already done one POST
     # that succeeded and now it failed - so retain the initial info
-    choices = config_value("TWO_FACTOR_ENABLED_METHODS")
-    if not config_value("TWO_FACTOR_REQUIRED"):
+    choices = cv("TWO_FACTOR_ENABLED_METHODS")
+    if not cv("TWO_FACTOR_REQUIRED"):
         choices.append("disable")
 
     if _security._want_json(request):
         # Provide information application/UI might need to render their own form/input
         json_response = {
-            "tf_required": config_value("TWO_FACTOR_REQUIRED"),
+            "tf_required": cv("TWO_FACTOR_REQUIRED"),
             "tf_primary_method": getattr(user, "tf_primary_method", None),
             "tf_phone_number": getattr(user, "tf_phone_number", None),
             "tf_available_methods": choices,
@@ -824,12 +821,12 @@ def two_factor_setup():
 
     code_form = _security.two_factor_verify_code_form()
     return _security.render_template(
-        config_value("TWO_FACTOR_SETUP_TEMPLATE"),
+        cv("TWO_FACTOR_SETUP_TEMPLATE"),
         two_factor_setup_form=form,
         two_factor_verify_code_form=code_form,
         choices=choices,
         chosen_method=form.setup.data,
-        two_factor_required=config_value("TWO_FACTOR_REQUIRED"),
+        two_factor_required=cv("TWO_FACTOR_REQUIRED"),
         **_ctx("tf_setup"),
     )
 
@@ -919,9 +916,9 @@ def two_factor_token_validation():
 
             return redirect(get_post_login_redirect())
 
-        if (
-            not config_value("TWO_FACTOR_ALWAYS_VALIDATE") and remember
-        ) and _security._want_json(request):
+        if (not cv("TWO_FACTOR_ALWAYS_VALIDATE") and remember) and _security._want_json(
+            request
+        ):
             token = generate_tf_validity_token(form.user.fs_uniquifier)
             json_response = {"tf_validity_token": token}
             return base_render_json(form, additional=json_response)
@@ -934,10 +931,10 @@ def two_factor_token_validation():
         setup_form = _security.two_factor_setup_form()
 
         return _security.render_template(
-            config_value("TWO_FACTOR_SETUP_TEMPLATE"),
+            cv("TWO_FACTOR_SETUP_TEMPLATE"),
             two_factor_setup_form=setup_form,
             two_factor_verify_code_form=form,
-            choices=config_value("TWO_FACTOR_ENABLED_METHODS"),
+            choices=cv("TWO_FACTOR_ENABLED_METHODS"),
             **_ctx("tf_setup"),
         )
 
@@ -946,7 +943,7 @@ def two_factor_token_validation():
         rescue_form = _security.two_factor_rescue_form()
 
         return _security.render_template(
-            config_value("TWO_FACTOR_VERIFY_CODE_TEMPLATE"),
+            cv("TWO_FACTOR_VERIFY_CODE_TEMPLATE"),
             two_factor_rescue_form=rescue_form,
             two_factor_verify_code_form=form,
             problem=None,
@@ -1007,8 +1004,8 @@ def two_factor_rescue():
         # send app provider a mail message regarding trouble
         elif problem == "no_mail_access":
             send_mail(
-                config_value("EMAIL_SUBJECT_TWO_FACTOR_RESCUE"),
-                config_value("TWO_FACTOR_RESCUE_MAIL"),
+                cv("EMAIL_SUBJECT_TWO_FACTOR_RESCUE"),
+                cv("TWO_FACTOR_RESCUE_MAIL"),
                 "two_factor_rescue",
                 user=form.user,
             )
@@ -1020,10 +1017,10 @@ def two_factor_rescue():
 
     code_form = _security.two_factor_verify_code_form()
     return _security.render_template(
-        config_value("TWO_FACTOR_VERIFY_CODE_TEMPLATE"),
+        cv("TWO_FACTOR_VERIFY_CODE_TEMPLATE"),
         two_factor_verify_code_form=code_form,
         two_factor_rescue_form=form,
-        rescue_mail=config_value("TWO_FACTOR_RESCUE_MAIL"),
+        rescue_mail=cv("TWO_FACTOR_RESCUE_MAIL"),
         problem=rproblem,
         **_ctx("tf_token_validation"),
     )
@@ -1062,13 +1059,13 @@ def create_blueprint(app, state, import_name, json_encoder=None):
             state.login_url + slash_url_suffix(state.login_url, "<token>"),
             endpoint="token_login",
         )(token_login)
-    elif config_value("US_SIGNIN_REPLACES_LOGIN", app=app):
+    elif cv("US_SIGNIN_REPLACES_LOGIN", app=app):
         bp.route(state.login_url, methods=["GET", "POST"], endpoint="login")(us_signin)
 
     else:
         bp.route(state.login_url, methods=["GET", "POST"], endpoint="login")(login)
 
-    if config_value("FRESHNESS", app=app).total_seconds() >= 0:
+    if cv("FRESHNESS", app=app).total_seconds() >= 0:
         bp.route(state.verify_url, methods=["GET", "POST"], endpoint="verify")(verify)
 
     if state.unified_signin:
@@ -1090,7 +1087,7 @@ def create_blueprint(app, state, import_name, json_encoder=None):
         )(us_setup_validate)
 
         # Freshness verification
-        if config_value("FRESHNESS", app=app).total_seconds() >= 0:
+        if cv("FRESHNESS", app=app).total_seconds() >= 0:
             bp.route(
                 state.us_verify_url, methods=["GET", "POST"], endpoint="us_verify"
             )(us_verify)
